@@ -1,3 +1,4 @@
+import copy
 import importlib
 import io
 import json
@@ -8,6 +9,7 @@ from collections.abc import Callable
 from datetime import datetime, timedelta
 from itertools import groupby
 from operator import itemgetter
+from urllib.parse import urlparse
 
 import pyarrow.parquet as pq
 import requests
@@ -19,10 +21,13 @@ from structlog import get_logger
 
 from eodash_catalog.sh_endpoint import get_SH_token
 from eodash_catalog.stac_handling import (
+    add_authentication,
+    add_base_overlay_info,
     add_collection_information,
     add_example_info,
     add_process_info_child_collection,
     add_projection_info,
+    add_raw_assets,
     get_collection_datetimes_from_config,
     get_or_create_collection,
 )
@@ -172,7 +177,7 @@ def handle_STAC_based_endpoint(
             catalog, collection_config["Name"], collection_config, catalog_config, endpoint_config
         )
         for location in collection_config["Locations"]:
-            identifier = location.get("Identifier", uuid.uuid4())
+            identifier = location.get("Identifier", str(uuid.uuid4()))
             collection = process_STACAPI_Endpoint(
                 catalog_config=catalog_config,
                 endpoint_config=endpoint_config,
@@ -205,18 +210,20 @@ def handle_STAC_based_endpoint(
             add_example_info(collection, collection_config, endpoint_config, catalog_config)
             # eodash v4 compatibility
             add_visualization_info(collection, collection_config, endpoint_config)
-            add_process_info_child_collection(collection, catalog_config, collection_config)
+            add_process_info_child_collection(collection, catalog_config, collection_config, None)
             if location.get("OverwriteBBox"):
                 collection.extent.spatial = SpatialExtent(
                     [
                         location["OverwriteBBox"],
                     ]
                 )
+            add_collection_information(catalog_config, collection, collection_config)
+            add_base_overlay_info(collection, catalog_config, collection_config)
         update_extents_from_collection_children(root_collection)
     else:
         bbox = None
-        if endpoint_config.get("Bbox"):
-            bbox = ",".join(map(str, endpoint_config["Bbox"]))
+        if endpoint_config.get("OverwriteBBox"):
+            bbox = ",".join(map(str, endpoint_config["OverwriteBBox"]))
         root_collection = process_STACAPI_Endpoint(
             catalog_config=catalog_config,
             endpoint_config=endpoint_config,
@@ -262,6 +269,7 @@ def process_STACAPI_Endpoint(
     api = Client.open(endpoint_config["EndPoint"], headers=headers)
     if bbox is None:
         bbox = [-180, -90, 180, 90]
+
     results = api.search(
         collections=[collection_id],
         bbox=bbox,
@@ -288,9 +296,13 @@ def process_STACAPI_Endpoint(
             else:
                 generate_thumbnail(item, collection_config, endpoint_config)
         # Check if we can create visualization link
-        if endpoint_config.get("Assets"):
+        if endpoint_config.get("Name") == "VEDA" and endpoint_config.get("Type") == "tiles":
             add_visualization_info(item, collection_config, endpoint_config, item.id)
-        elif item.assets.get("cog_default"):
+        elif (
+            endpoint_config.get("Name") == "VEDA"
+            and endpoint_config.get("Type") == "cog"
+            and item.assets.get("cog_default")
+        ):
             add_visualization_info(
                 item, collection_config, endpoint_config, item.assets["cog_default"].href
             )
@@ -391,6 +403,7 @@ def handle_collection_only(
                 properties={},
                 geometry=None,
                 datetime=dt,
+                assets={"dummy_asset": Asset(href="")},
             )
             link = collection.add_item(item)
             link.extra_fields["datetime"] = format_datetime_to_isostring_zulu(dt)
@@ -462,13 +475,15 @@ def handle_SH_WMS_endpoint(
             elif not location["Times"]:
                 LOGGER.warn(f"NO datetimes configured for collection: {collection_config['Name']}!")
             add_visualization_info(collection, collection_config, endpoint_config)
-            add_process_info_child_collection(collection, catalog_config, collection_config)
+            add_process_info_child_collection(collection, catalog_config, collection_config, None)
+            add_collection_information(catalog_config, collection, collection_config)
+            add_base_overlay_info(collection, catalog_config, collection_config)
         update_extents_from_collection_children(root_collection)
     else:
         # if locations are not provided, treat the collection as a
         # general proxy to the sentinel hub layer
         datetimes = get_collection_datetimes_from_config(endpoint_config)
-        bbox = endpoint_config.get("Bbox", [-180, -85, 180, 85])
+        bbox = endpoint_config.get("OverwriteBBox", [-180, -85, 180, 85])
         items = []
         for dt in datetimes:
             item = Item(
@@ -538,6 +553,7 @@ def handle_rasdaman_endpoint(
     # add_example_info(collection, collection_config, endpoint_config, catalog_config)
     return collection
 
+
 def handle_GeoDB_Features_endpoint(
     catalog_config: dict,
     endpoint_config: dict,
@@ -546,7 +562,6 @@ def handle_GeoDB_Features_endpoint(
     catalog: Catalog,
     options: Options,
 ) -> Collection:
-
     # ID of collection is data["Name"] instead of CollectionId to be able to
     # create more STAC collections from one geoDB table
     collection = get_or_create_collection(
@@ -581,13 +596,9 @@ def handle_GeoDB_Features_endpoint(
                     datetime(time_object.year, time_object.month, time_object.day).date()
                 )
             case "month":
-                unique_datetimes.add(
-                    datetime(time_object.year, time_object.month, 1).date()
-                )
+                unique_datetimes.add(datetime(time_object.year, time_object.month, 1).date())
             case "year":
-                unique_datetimes.add(
-                    datetime(time_object.year, 1, 1).date()
-                )
+                unique_datetimes.add(datetime(time_object.year, 1, 1).date())
             case _:
                 # default to day
                 unique_datetimes.add(
@@ -610,10 +621,11 @@ def handle_GeoDB_Features_endpoint(
         updated_query = endpoint_config["Query"].replace("{{date_time}}", matching_string)
         assets = {
             "geodbfeatures": Asset(
-            href=f"{endpoint_config['EndPoint']}{endpoint_config['Database']}_{endpoint_config['CollectionId']}?{updated_query}",
-            media_type="application/geodb+json",
-            roles=["data"],
-        )}
+                href=f"{endpoint_config['EndPoint']}{endpoint_config['Database']}_{endpoint_config['CollectionId']}?{updated_query}",
+                media_type="application/geodb+json",
+                roles=["data"],
+            )
+        }
         item = Item(
             id=format_datetime_to_isostring_zulu(item_datetime),
             bbox=endpoint_config.get("OverwriteBBox", [-180, -90, 180, 90]),
@@ -708,13 +720,13 @@ def handle_GeoDB_endpoint(
         latlon = unique_values["aoi"]
         [lat, lon] = [float(x) for x in latlon.split(",")]
         # create item for unique locations
-        buff = 0.01
+        buff = 0.2
         bbox = [lon - buff, lat - buff, lon + buff, lat + buff]
 
         # create collection per available inputdata information
         sc_config = {
-            "Title": city,
-            "Description": f"{city} - {country}",
+            "Title": f"{city} - {collection_config['Name']}",
+            "Description": collection_config["Description"],
         }
         locations_collection = get_or_create_collection(
             collection, key, sc_config, catalog_config, endpoint_config
@@ -724,7 +736,33 @@ def handle_GeoDB_endpoint(
             input_data = []
         if len(input_data) > 0 or endpoint_config.get("FeatureCollection"):
             items = []
-            for v in values:
+            content_for_individual_datetimes = values
+            if endpoint_config.get("MapTimesCollection"):
+                # extract datetimes from another table if configured so and match it based on aoi_id
+                # special for E13d
+                select = f"?select=time&aoi_id=eq.{key}"
+                url = (
+                    endpoint_config["EndPoint"]
+                    + endpoint_config["Database"]
+                    + "_{}".format(endpoint_config["MapTimesCollection"])
+                    + select
+                )
+                response = json.loads(requests.get(url).text)
+                content_for_individual_datetimes = []
+                for response_obj in response:
+                    time_object = datetime.fromisoformat(response_obj["time"])
+                    for searched_row in values:
+                        search_datetime = datetime.fromisoformat(searched_row["time"])
+                        if (
+                            search_datetime.month == time_object.month
+                            and search_datetime.year == time_object.year
+                        ):
+                            break
+                    insert_row = copy.deepcopy(searched_row)
+                    # overwrite time with one from another collection and save
+                    insert_row["time"] = response_obj["time"]
+                    content_for_individual_datetimes.append(insert_row)
+            for v in content_for_individual_datetimes:
                 # add items based on inputData fields for each time step available in values
                 first_match: dict = next(
                     (item for item in input_data if item.get("Identifier") == v["input_data"]), None
@@ -756,8 +794,16 @@ def handle_GeoDB_endpoint(
 
                 assets = {"dummy_asset": Asset(href="")}
                 if endpoint_config.get("FeatureCollection"):
+                    url = f"{endpoint_config['EndPoint']}{endpoint_config['Database']}_{endpoint_config['FeatureCollection']}?aoi_id=eq.{v['aoi_id']}"  # noqa: E501
+                    if collection_config.get("EodashIdentifier") == "E13d":
+                        # custom override of E13d to be +- 60 minutes around observation datetime
+                        time_start_features = (time_object - timedelta(minutes=60)).isoformat()
+                        time_end_features = (time_object + timedelta(minutes=60)).isoformat()
+                        url += f"&time=gte.{time_start_features}&time=lte.{time_end_features}"
+                    else:
+                        url += f"&time=eq.{v['time']}"
                     assets["geodbfeatures"] = Asset(
-                        href=f"{endpoint_config['EndPoint']}{endpoint_config['Database']}_{endpoint_config['FeatureCollection']}?aoi_id=eq.{v['aoi_id']}&time=eq.{v['time']}",
+                        href=url,
                         media_type="application/geodb+json",
                         roles=["data"],
                     )
@@ -813,7 +859,7 @@ def handle_GeoDB_endpoint(
                                 rel="wms",
                                 target=url,
                                 media_type=(endpoint_config.get("MimeType", "image/png")),
-                                title=collection_config["Name"],
+                                title=first_match["Identifier"],
                                 extra_fields=extra_fields,
                             )
                             item.add_link(link)
@@ -877,6 +923,9 @@ def handle_GeoDB_endpoint(
             individual_datetimes = [datetime.fromisoformat(v["time"]) for v in values]
             time_extent = [min(individual_datetimes), max(individual_datetimes)]
             locations_collection.extent.temporal = TemporalExtent([time_extent])
+        add_process_info_child_collection(
+            locations_collection, catalog_config, collection_config, key
+        )
         locations_collection.extra_fields["subcode"] = key
         link = collection.add_child(locations_collection)
         # collection.update_extent_from_items()
@@ -886,6 +935,7 @@ def handle_GeoDB_endpoint(
         link.extra_fields["country"] = country
         link.extra_fields["name"] = city
         add_collection_information(catalog_config, locations_collection, collection_config)
+        add_base_overlay_info(locations_collection, catalog_config, collection_config)
 
     if "yAxis" not in collection_config:
         # fetch yAxis and store it to data, preventing need to save it per dataset in yml
@@ -924,7 +974,8 @@ def handle_SH_endpoint(
 ) -> Collection:
     token = get_SH_token(endpoint_config)
     headers = {"Authorization": f"Bearer {token}"}
-    endpoint_config["EndPoint"] = "https://services.sentinel-hub.com/api/v1/catalog/1.0.0/"
+    endpoint_url_parts = urlparse(endpoint_config["EndPoint"])
+    endpoint_config["EndPoint"] = f"https://{endpoint_url_parts.netloc}/api/v1/catalog/1.0.0/"
     # Overwrite collection id with type, such as ZARR or BYOC
     if endpoint_config.get("Type"):
         endpoint_config["CollectionId"] = (
@@ -961,12 +1012,14 @@ def handle_WMS_endpoint(
         # some endpoints allow "narrowed-down" capabilities per-layer, which we utilize to not
         # have to process full service capabilities XML
         capabilities_url = endpoint_config["EndPoint"]
-        spatial_extent, datetimes = retrieveExtentFromWMSWMTS(
+        spatial_extent, datetimes_retrieved = retrieveExtentFromWMSWMTS(
             capabilities_url,
             endpoint_config["LayerId"],
             version=endpoint_config.get("Version", "1.1.1"),
             wmts=wmts,
         )
+        if datetimes_retrieved:
+            datetimes = datetimes_retrieved
     # optionally filter time results
     if query := endpoint_config.get("Query"):
         datetimes = filter_time_entries(datetimes, query)
@@ -1027,9 +1080,25 @@ def generate_veda_tiles_link(endpoint_config: dict, item: str | None) -> str:
     assets = ""
     for asset in endpoint_config["Assets"]:
         assets += f"&assets={asset}"
+
+    colormap_name = ""
+    if endpoint_config.get("ColormapName"):
+        colormap_name = "&colormap_name={}".format(endpoint_config["ColormapName"])
+
     color_formula = ""
     if endpoint_config.get("ColorFormula"):
         color_formula = "&color_formula={}".format(endpoint_config["ColorFormula"])
+    rescale = ""
+    if rescale_configs := endpoint_config.get("Rescale", ""):
+        if isinstance(rescale_configs[0], list):
+            # one rescale definition for each band
+            for rescale_config in rescale_configs:
+                rescale += f"&rescale={rescale_config[0]},{rescale_config[1]}"
+        else:
+            # shared rescale definition for all bands
+            rescale = "&rescale={},{}".format(
+                endpoint_config["Rescale"][0], endpoint_config["Rescale"][1]
+            )
     no_data = ""
     if endpoint_config.get("NoData"):
         no_data = "&no_data={}".format(endpoint_config["NoData"])
@@ -1037,7 +1106,7 @@ def generate_veda_tiles_link(endpoint_config: dict, item: str | None) -> str:
     target_url_base = endpoint_config["EndPoint"].replace("/stac/", "")
     target_url = (
         f"{target_url_base}/raster/collections/{collection}/items/{item}"
-        f"/tiles/WebMercatorQuad/{{z}}/{{x}}/{{y}}?{assets}{color_formula}{no_data}"
+        f"/tiles/WebMercatorQuad/{{z}}/{{x}}/{{y}}?{assets}{colormap_name}{color_formula}{no_data}{rescale}"
     )
     return target_url
 
@@ -1087,9 +1156,10 @@ def add_visualization_info(
 
         if dimensions != {}:
             extra_fields["wms:dimensions"] = dimensions
+        endpoint_url_parts = urlparse(endpoint_config["EndPoint"])
         link = Link(
             rel="wms",
-            target=f"https://services.sentinel-hub.com/ogc/wms/{instanceId}",
+            target=f"https://{endpoint_url_parts.netloc}/ogc/wms/{instanceId}",
             media_type=(endpoint_config.get("MimeType", "image/png")),
             title=collection_config["Name"],
             extra_fields=extra_fields,
@@ -1106,6 +1176,14 @@ def add_visualization_info(
                 "role": ["data"],
             }
         )
+        if collection_config.get("EodashIdentifier") == "FNF":
+            extra_fields.update(
+                {
+                    "wms:layers": endpoint_config.get("LayerId", "").replace(
+                        "{time}", (datetimes is not None and str(datetimes[0].year)) or "2020"
+                    ),
+                }
+            )
         dimensions = {}
         if dimensions_config := endpoint_config.get("Dimensions", {}):
             for key, value in dimensions_config.items():
@@ -1179,28 +1257,6 @@ def add_visualization_info(
             link,
         )
         stac_object.add_link(link)
-    elif endpoint_config["Name"] == "JAXA_WMTS_PALSAR":
-        target_url = "{}".format(endpoint_config.get("EndPoint"))
-        # custom time just for this special case as a default for collection wmts
-        time = None
-        if datetimes is not None:
-            time = datetimes[0]
-        extra_fields.update(
-            {
-                "wmts:layer": endpoint_config.get("LayerId", "").replace(
-                    "{time}", (time and str(time.year)) or "2017"
-                )
-            }
-        )
-        stac_object.add_link(
-            Link(
-                rel="wmts",
-                target=target_url,
-                media_type="image/png",
-                title="wmts capabilities",
-                extra_fields=extra_fields,
-            )
-        )
     elif endpoint_config["Name"] == "xcube":
         if endpoint_config["Type"] == "zarr":
             # either preset ColormapName of left as a template
@@ -1368,22 +1424,10 @@ def handle_raw_source(
         items = []
         style_link = None
         for time_entry in endpoint_config["TimeEntries"]:
-            assets = {}
-            media_type = "application/geo+json"
-            style_type = "text/vector-styles"
-            if endpoint_config["Name"] == "COG source":
-                style_type = "text/cog-styles"
-                media_type = "image/tiff"
-            if endpoint_config["Name"] == "FlatGeobuf source":
-                media_type = "application/vnd.flatgeobuf"
-            for a in time_entry["Assets"]:
-                asset = Asset(
-                    href=a["File"], roles=["data"], media_type=media_type, extra_fields={}
-                )
-                add_projection_info(endpoint_config, asset)
-                assets[a["Identifier"]] = asset
+            assets, style_link = add_raw_assets(time_entry, endpoint_config, catalog_config, "data")
             bbox = endpoint_config.get("Bbox", [-180, -85, 180, 85])
             dt = parse_datestring_to_tz_aware_datetime(time_entry["Time"])
+
             item = Item(
                 id=format_datetime_to_isostring_zulu(dt),
                 bbox=bbox,
@@ -1397,22 +1441,15 @@ def handle_raw_source(
                 item.stac_extensions.append(
                     "https://stac-extensions.github.io/attribution/v0.1.0/schema.json"
                 )
-                asset.extra_fields["attribution"] = endpoint_config["Attribution"]
+                item.extra_fields["attribution"] = endpoint_config["Attribution"]
+            if "MergeAssets" in endpoint_config:
+                item.extra_fields["merge_assets"] = endpoint_config["MergeAssets"]
+
             add_projection_info(
                 endpoint_config,
                 item,
             )
-            if ep_st := endpoint_config.get("Style"):
-                style_link = Link(
-                    rel="style",
-                    target=ep_st
-                    if ep_st.startswith("http")
-                    else f"{catalog_config['assets_endpoint']}/{ep_st}",
-                    media_type=style_type,
-                    extra_fields={
-                        "asset:keys": list(assets),
-                    },
-                )
+            if style_link:
                 item.add_link(style_link)
             items.append(item)
 
@@ -1459,3 +1496,93 @@ def handle_raw_source(
 
     add_collection_information(catalog_config, collection, collection_config)
     return collection
+
+
+def handle_vector_tile_source(
+    catalog_config: dict,
+    endpoint_config: dict,
+    collection_config: dict,
+    coll_path_rel_to_root_catalog: str,
+    catalog: Catalog,
+    options: Options,
+) -> Collection:
+    collection = get_or_create_collection(
+        catalog, collection_config["Name"], collection_config, catalog_config, endpoint_config
+    )
+    coll_path_rel_to_root_catalog = f'{coll_path_rel_to_root_catalog}/{collection_config["Name"]}'
+    if len(endpoint_config.get("TimeEntries", [])) > 0:
+        items = []
+        style_link = None
+        for time_entry in endpoint_config["TimeEntries"]:
+            # create Item for each time entry
+            media_type = "application/vnd.mapbox-vector-tile"
+            style_type = "text/vector-styles"
+            bbox = endpoint_config.get("Bbox", [-180, -85, 180, 85])
+            dt = parse_datestring_to_tz_aware_datetime(time_entry["Time"])
+
+            item = Item(
+                id=format_datetime_to_isostring_zulu(dt),
+                bbox=bbox,
+                properties={},
+                geometry=create_geometry_from_bbox(bbox),
+                datetime=dt,
+                extra_fields={},
+                assets={"dummy_asset": Asset(href="")},
+            )
+            extra_fields_link = {}
+            add_authentication(item, time_entry["Url"], extra_fields_link)
+            # add mapbox vector tile link
+            identifier = str(uuid.uuid4())
+            extra_fields_link["key"] = identifier
+            if vector_tile_id_property := endpoint_config.get("idProperty"):
+                extra_fields_link["idProperty"] = vector_tile_id_property
+            if vector_tile_id_property := endpoint_config.get("layers"):
+                extra_fields_link["layers"] = vector_tile_id_property
+            link = Link(
+                rel="vector-tile",
+                target=time_entry["Url"],
+                media_type=media_type,
+                title=collection_config["Name"],
+                extra_fields=extra_fields_link,
+            )
+            add_projection_info(
+                endpoint_config,
+                link,
+            )
+            item.add_link(link)
+            add_projection_info(
+                endpoint_config,
+                item,
+            )
+            if endpoint_config.get("Attribution"):
+                item.stac_extensions.append(
+                    "https://stac-extensions.github.io/attribution/v0.1.0/schema.json"
+                )
+                item.extra_fields["attribution"] = endpoint_config["Attribution"]
+            # add style
+            if ep_st := endpoint_config.get("Style"):
+                style_link = Link(
+                    rel="style",
+                    target=ep_st
+                    if ep_st.startswith("http")
+                    else f"{catalog_config['assets_endpoint']}/{ep_st}",
+                    media_type=style_type,
+                    extra_fields={"links:keys": [identifier]},
+                )
+                item.add_link(style_link)
+            items.append(item)
+
+        save_items(
+            collection,
+            items,
+            options.outputpath,
+            catalog_config["id"],
+            coll_path_rel_to_root_catalog,
+            options.gp,
+        )
+
+    else:
+        LOGGER.warn(f"NO datetimes configured for collection: {collection_config['Name']}!")
+
+    add_collection_information(catalog_config, collection, collection_config)
+    return collection
