@@ -24,6 +24,7 @@ from eodash_catalog.endpoints import (
     handle_raw_source,
     handle_SH_endpoint,
     handle_SH_WMS_endpoint,
+    handle_vector_tile_source,
     handle_VEDA_endpoint,
     handle_WMS_endpoint,
     handle_xcube_endpoint,
@@ -292,16 +293,14 @@ def process_collection_file(
                         catalog,
                         options,
                     )
-                elif endpoint_config["Name"] == "JAXA_WMTS_PALSAR":
-                    # somewhat one off creation of individual WMTS layers as individual items
-                    collection = handle_WMS_endpoint(
+                elif endpoint_config["Name"] == "VectorTile source":
+                    collection = handle_vector_tile_source(
                         catalog_config,
                         endpoint_config,
                         collection_config,
                         coll_path_rel_to_root_catalog,
                         catalog,
                         options,
-                        wmts=True,
                     )
                 elif endpoint_config["Name"] == "Collection-only":
                     collection = handle_collection_only(
@@ -360,9 +359,11 @@ def process_collection_file(
                     countries.extend(sub_coll_def["Country"])
                 else:
                     countries.append(sub_coll_def["Country"])
-                coll_path_rel_to_root_catalog = (
-                    f"{coll_path_rel_to_root_catalog}/{sub_coll_def['Collection']}"
-                )
+                # commented out intentionally, because otherwise paths further down did
+                # not match, parquet file was one level deeper
+                # coll_path_rel_to_root_catalog = (
+                #     f"{coll_path_rel_to_root_catalog}/{sub_coll_def['Collection']}"
+                # )
                 process_collection_file(
                     catalog_config,
                     "{}/{}".format(options.collectionspath, sub_coll_def["Collection"]),
@@ -421,6 +422,7 @@ def process_collection_file(
         add_collection_information(catalog_config, parent_collection, collection_config, True)
         add_process_info(parent_collection, catalog_config, collection_config)
         update_extents_from_collection_children(parent_collection)
+        add_base_overlay_info(parent_collection, catalog_config, collection_config)
         # Fill summaries for locations
         parent_collection.summaries = Summaries(
             {
@@ -456,12 +458,20 @@ def add_to_catalog(
         link.extra_fields["endpointtype"] = endpoint["Name"]
     if collection_config.get("Subtitle"):
         link.extra_fields["subtitle"] = collection_config["Subtitle"]
+    if collection_config.get("ShortDescription"):
+        link.extra_fields["shortdescription"] = collection_config["ShortDescription"]
     link.extra_fields["title"] = collection.title
     if collection_config.get("EodashIdentifier"):
         link.extra_fields["code"] = collection_config["EodashIdentifier"]
     link.extra_fields["id"] = collection_config["Name"]
     if collection_config.get("Themes"):
         link.extra_fields["themes"] = collection_config["Themes"]
+    if collection_config.get("Provider"):
+        # get all provider names
+        link.extra_fields["providers"] = [
+            provider.get("Name") or provider.get("Url")
+            for provider in collection_config["Provider"]
+        ]
     # Check for summaries and bubble up info
     if disable:
         link.extra_fields["roles"] = ["disable"]
