@@ -1,4 +1,6 @@
+import uuid
 from datetime import datetime, timezone
+from urllib.parse import parse_qs, urlparse
 
 import requests
 import spdx_lookup as lookup
@@ -86,7 +88,9 @@ def get_or_create_collection(
     return collection
 
 
-def create_service_link(endpoint_config: dict, catalog_config: dict) -> Link:
+def create_service_link(
+    endpoint_config: dict, catalog_config: dict, location_id: str | None = None
+) -> Link:
     extra_fields = {
         "id": endpoint_config["Identifier"],
         "method": endpoint_config.get("Method", "GET"),
@@ -113,20 +117,26 @@ def create_service_link(endpoint_config: dict, catalog_config: dict) -> Link:
                 extra_fields["eox:flatstyle"].append(flatstyle_obj)
         else:
             LOGGER.warn("Flatstyle is invalid type", endpoint_config["Flatstyle"])
+    url = endpoint_config["Url"]
+    if location_id:
+        url = url.replace("{{feature}}", location_id)
     sl = Link(
         rel="service",
-        target=endpoint_config["Url"],
+        target=url,
         media_type=endpoint_config["Type"],
         extra_fields=extra_fields,
     )
     return sl
 
 
-def create_web_map_link(layer_config: dict, role: str) -> Link:
+def create_web_map_link(
+    collection: Collection, catalog_config: dict, layer_config: dict, role: str
+) -> Link:
     extra_fields = {
         "roles": [role],
         "id": layer_config["id"],
     }
+    media_type = layer_config.get("media_type", "image/png")
     if layer_config.get("default"):
         extra_fields["roles"].append("default")
     if layer_config.get("visible"):
@@ -146,12 +156,34 @@ def create_web_map_link(layer_config: dict, role: str) -> Link:
             extra_fields["wmts:layer"] = layer_config["layer"]
             if layer_config.get("dimensions"):
                 extra_fields["wmts:dimensions"] = layer_config["dimensions"]
+        case "vector-tile":
+            identifier = str(uuid.uuid4())
+            extra_fields["key"] = identifier
+            media_type = "application/vnd.mapbox-vector-tile"
+            if vector_tile_id_property := layer_config.get("idProperty"):
+                extra_fields["idProperty"] = vector_tile_id_property
+            if vector_tile_id_property := layer_config.get("layers"):
+                layer_config["layers"] = vector_tile_id_property
+            if ep_st := layer_config.get("Style"):
+                style_link = Link(
+                    rel="style",
+                    target=ep_st
+                    if ep_st.startswith("http")
+                    else f"{catalog_config['assets_endpoint']}/{ep_st}",
+                    media_type="text/vector-styles",
+                    extra_fields={"links:keys": [identifier]},
+                )
+                collection.add_link(style_link)
+            add_authentication(collection, layer_config["url"], extra_fields)
+
     if layer_config.get("Attribution"):
         extra_fields["attribution"] = layer_config["Attribution"]
+    if layer_config.get("Colorlegend"):
+        extra_fields["eox:colorlegend"] = layer_config["Colorlegend"]
     wml = Link(
         rel=layer_config["protocol"],
         target=layer_config["url"],
-        media_type=layer_config.get("media_type", "image/png"),
+        media_type=media_type,
         title=layer_config["name"],
         extra_fields=extra_fields,
     )
@@ -159,6 +191,52 @@ def create_web_map_link(layer_config: dict, role: str) -> Link:
     return wml
 
 
+def add_raw_assets(time_entry: dict, endpoint_config: dict, catalog_config: dict, role: str):
+    assets = {}
+    style_link = None
+    media_type = "application/geo+json"
+    style_type = "text/vector-styles"
+    extra_fields_asset = {}
+    roles = [role]
+    if endpoint_config.get("visible"):
+        roles.append("visible")
+    if "visible" in endpoint_config and not endpoint_config["visible"]:
+        roles.append("invisible")
+
+    if endpoint_config.get("Attribution"):
+        extra_fields_asset["attribution"] = endpoint_config["Attribution"]
+    if endpoint_config.get("Colorlegend"):
+        extra_fields_asset["eox:colorlegend"] = endpoint_config["Colorlegend"]
+
+    if endpoint_config["Name"] == "COG source":
+        style_type = "text/cog-styles"
+        media_type = "image/tiff"
+    if endpoint_config["Name"] == "FlatGeobuf source":
+        media_type = "application/vnd.flatgeobuf"
+    for a in time_entry["Assets"]:
+        asset = Asset(
+            href=a["File"],
+            title=a.get("Title", a["Identifier"]),
+            roles=roles,
+            media_type=media_type,
+            extra_fields=extra_fields_asset,
+        )
+        add_projection_info(endpoint_config, asset)
+        assets[a["Identifier"]] = asset
+    if ep_st := endpoint_config.get("Style"):
+        style_link = Link(
+            rel="style",
+            target=ep_st
+            if ep_st.startswith("http")
+            else f"{catalog_config['assets_endpoint']}/{ep_st}",
+            media_type=style_type,
+            extra_fields={
+                "asset:keys": list(assets),
+            },
+        )
+    return assets, style_link
+
+
 def add_example_info(
     stac_object: Collection | Catalog,
     collection_config: dict,
@@ -288,6 +366,10 @@ def add_collection_information(
 
     if collection_config.get("Subtitle"):
         collection.extra_fields["subtitle"] = collection_config["Subtitle"]
+
+    if collection_config.get("ShortDescription"):
+        collection.extra_fields["shortdescription"] = collection_config["ShortDescription"]
+
     if collection_config.get("Legend"):
         collection.add_asset(
             "legend",
@@ -297,28 +379,42 @@ def add_collection_information(
                 roles=["metadata"],
             ),
         )
-    if collection_config.get("Story"):
-        collection.add_asset(
-            "story",
-            Asset(
-                href=f'{catalog_config["assets_endpoint"]}/{collection_config["Story"]}',
-                media_type="text/markdown",
-                roles=["metadata"],
-            ),
-        )
+    if stories := collection_config.get("Stories"):
+        for story in stories:
+            story_url = story.get("Url")
+            if not story_url.startswith("http"):
+                story_url = f'{catalog_config.get("stories_endpoint")}/{story_url}'
+            parsed_url = urlparse(story_url)
+            # check if it is URL with a query parameter id=story-identifier
+            if parsed_url.query and len(parse_qs(parsed_url.query).get("id")) > 0:
+                story_id = parse_qs(parsed_url.query).get("id")[0]
+            else:
+                story_id = parsed_url.path.rsplit("/")[-1].replace(".md", "").replace(".MD", "")
+            collection.add_asset(
+                story_id,
+                Asset(
+                    title=story.get("Name"),
+                    href=story_url,
+                    media_type="text/markdown",
+                    roles=["metadata", "story"],
+                ),
+            )
     if collection_config.get("Image"):
+        # Check if absolute URL or relative path
+        if collection_config["Image"].startswith("http"):
+            image_url = collection_config["Image"]
+        else:
+            image_url = f'{catalog_config["assets_endpoint"]}/{collection_config["Image"]}'
         collection.add_asset(
             "thumbnail",
             Asset(
-                href=f'{catalog_config["assets_endpoint"]}/{collection_config["Image"]}',
+                href=image_url,
                 media_type="image/png",
                 roles=["thumbnail"],
             ),
         )
         # Bubble up thumbnail to extra fields
-        collection.extra_fields["thumbnail"] = (
-            f'{catalog_config["assets_endpoint"]}/' f'{collection_config["Image"]}'
-        )
+        collection.extra_fields["thumbnail"] = image_url
     # Add extra fields to collection if available
     add_extra_fields(collection, collection_config, is_root_collection)
 
@@ -452,17 +548,25 @@ def add_process_info(collection: Collection, catalog_config: dict, collection_co
 
 
 def add_process_info_child_collection(
-    collection: Collection, catalog_config: dict, collection_config: dict
+    collection: Collection, catalog_config: dict, collection_config: dict, location_id: str | None
 ) -> None:
     # in case of locations, we add the process itself on a child collection
     if collection_config.get("Process"):
         if collection_config["Process"].get("EndPoints"):
             for endpoint in collection_config["Process"]["EndPoints"]:
-                collection.add_link(create_service_link(endpoint, catalog_config))
+                link = create_service_link(endpoint, catalog_config, location_id)
+                collection.add_link(link)
         if collection_config["Process"].get("JsonForm"):
-            collection.extra_fields["eodash:jsonform"] = get_full_url(
-                collection_config["Process"]["JsonForm"], catalog_config
-            )
+            if location_id and catalog_config.get("geodb_empty_form"):
+                # specific handling of geodb locations to replace with an empty jsonform
+                collection.extra_fields["eodash:jsonform"] = get_full_url(
+                    catalog_config["geodb_empty_form"], catalog_config
+                )
+            else:
+                # standard json form pass through
+                collection.extra_fields["eodash:jsonform"] = get_full_url(
+                    collection_config["Process"]["JsonForm"], catalog_config
+                )
         if collection_config["Process"].get("VegaDefinition"):
             collection.extra_fields["eodash:vegadefinition"] = get_full_url(
                 collection_config["Process"]["VegaDefinition"], catalog_config
@@ -473,23 +577,52 @@ def add_base_overlay_info(
     collection: Collection, catalog_config: dict, collection_config: dict
 ) -> None:
     # add custom baselayers specially for this indicator
-    if collection_config.get("BaseLayers"):
-        for layer in collection_config["BaseLayers"]:
-            collection.add_link(create_web_map_link(layer, role="baselayer"))
     # alternatively use default base layers defined
-    elif catalog_config.get("default_base_layers"):
-        base_layers = read_config_file(catalog_config["default_base_layers"])
-        for layer in base_layers:
-            collection.add_link(create_web_map_link(layer, role="baselayer"))
+    if "BaseLayers" in collection_config or catalog_config.get("default_base_layers"):
+        layers = collection_config.get("BaseLayers") or read_config_file(
+            catalog_config["default_base_layers"]
+        )
+        for layer in layers:
+            if layer.get("Name") in [
+                "COG source",
+                "GeoJSON source",
+                "FlatGeobuf source",
+            ]:
+                time_entry_structure = {"Assets": [layer]}
+                assets, style_link = add_raw_assets(
+                    time_entry_structure, layer, catalog_config, role="baselayer"
+                )
+                for asset_key, asset in assets.items():
+                    collection.add_asset(asset_key, asset)
+                collection.add_link(style_link)
+                collection.extra_fields["merge_assets"] = False
+            else:
+                link = create_web_map_link(collection, catalog_config, layer, role="baselayer")
+                collection.add_link(link)
     # add custom overlays just for this indicator
-    if collection_config.get("OverlayLayers"):
-        for layer in collection_config["OverlayLayers"]:
-            collection.add_link(create_web_map_link(layer, role="overlay"))
-    # check if default overlay layers defined
-    elif catalog_config.get("default_overlay_layers"):
-        overlay_layers = read_config_file(catalog_config["default_overlay_layers"])
-        for layer in overlay_layers:
-            collection.add_link(create_web_map_link(layer, role="overlay"))
+    # alternatively use default overlays defined
+    if "OverlayLayers" in collection_config or catalog_config.get("default_overlay_layers"):
+        layers = collection_config.get("OverlayLayers") or read_config_file(
+            catalog_config["default_overlay_layers"]
+        )
+
+        for layer in layers:
+            if layer.get("Name") in [
+                "COG source",
+                "GeoJSON source",
+                "FlatGeobuf source",
+            ]:
+                time_entry_structure = {"Assets": [layer]}
+                assets, style_link = add_raw_assets(
+                    time_entry_structure, layer, catalog_config, role="overlay"
+                )
+                for asset_key, asset in assets.items():
+                    collection.add_asset(asset_key, asset)
+                collection.extra_fields["merge_assets"] = False
+                collection.add_link(style_link)
+            else:
+                link = create_web_map_link(collection, catalog_config, layer, role="overlay")
+                collection.add_link(link)
 
 
 def add_extra_fields(
@@ -579,3 +712,20 @@ def add_projection_info(
             stac_object.extra_fields["eodash:proj4_def"] = proj
         else:
             raise Exception(f"Incorrect type of proj definition {proj}")
+
+
+def add_authentication(stac_object: Item | Collection | Catalog, url: str, extra_fields_link: dict):
+    if "mapbox" in url:
+        # add authentication info
+        auth_extension = "https://stac-extensions.github.io/authentication/v1.1.0/schema.json"
+        if auth_extension not in stac_object.stac_extensions:
+            stac_object.stac_extensions.append(auth_extension)
+        stac_object.extra_fields["auth:schemes"] = {
+            "mapboxauth": {
+                "type": "apiKey",
+                "name": "access_token",
+                "in": "query",
+            }
+        }
+        extra_fields_link["auth:refs"] = ["mapboxauth"]
+    pass
